/*
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.phansson.example;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.InvocationTargetException;
import java.net.Authenticator;
import java.net.MalformedURLException;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.URL;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.phansson.dummyproxy.DummyProxyClientThread;

/**
 *  Demonstrates JDK-806818 using a simple Swing application.
 *  https://bitbucket.org/phansson/dummy-proxy-server
 */
public class JDKBug806818Example {
    
    private static JFrame frame;

    private static void createAndShowGUI() {
        frame = new JFrame("JDK Bug 806818 Example");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
 
        JButton btn = new JButton("Push me");
        btn.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) { 
                JOptionPane.showMessageDialog(frame, "I'm alive!", "Hello", JOptionPane.INFORMATION_MESSAGE);
            }
        });
        frame.getContentPane().add(btn);
           
        //Display the window.
        frame.pack();
        frame.setSize(new Dimension(800, 600));
        frame.setVisible(true);
    }
        
    public static String getPassword() {
        ClassLoader cloader = Thread.currentThread().getContextClassLoader();

        try {
            Class<?> loadedClass = cloader.loadClass("Whatever");
            System.out.println(loadedClass.getName());
        } catch (ClassNotFoundException ex) {
        }
        return (String)JOptionPane.showInputDialog(
                    frame,
                    "Password : ",
                    "Credentials for " + System.getProperty("user.name"),
                    JOptionPane.PLAIN_MESSAGE,
                    null,
                    null,
                    null);
    }
 
    
    public static void main(String[] args) throws MalformedURLException, IOException, InterruptedException, InvocationTargetException {
        final CountDownLatch started = new CountDownLatch(1);
         
        new Thread() {
            public void run() {
                int threadNo = 0;
                ExecutorService threadPool = Executors.newFixedThreadPool(10);
                try {
                    ServerSocket serverSocket = new ServerSocket(0);
                    System.setProperty("http.proxyHost", "localhost");
                    System.setProperty("http.proxyPort", String.valueOf(serverSocket.getLocalPort()));
                    System.setProperty("java.security.krb5.realm", "whatever");
                    System.setProperty("java.security.krb5.kdc", "whatever");
                    System.out.println("Dummy Proxy Server started");
                    started.countDown();
                    while (true) {
                        Socket clientSocket = serverSocket.accept();
                        threadNo++;
                        threadPool.submit(new DummyProxyClientThread(clientSocket, threadNo));
                    }

                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }.start();
        
        started.await();
        
        // Install our custom class loader
        Thread.currentThread().setContextClassLoader(new ClassLoader() {});
        
        // Install our simple Authenticator
        Authenticator.setDefault(new MyAuthenticator());
        
        // Show GUI
        javax.swing.SwingUtilities.invokeAndWait(new Runnable() {
            @Override
            public void run() {
                createAndShowGUI();
            }
        });

        URL url = new URL("http://netbeans.apache.org/");
        
        // this will now cause the deadlock
        try (InputStream is = url.openStream()) {
            // read from stream and discard
            byte[] buffer = new byte[1024];
            while (is.read(buffer) != -1) {}
        }
    }
    
}
